% CODING_LENGTH_CHAIN computes how bits are needed to represent the
% boundary of a region in an image using an adaptive chain code.
%
%   codingLength = coding_length_chain(frequencies, chainCodeBitCounts, pixelBitCount);
%
%   frequencies is an (n x 8) matrix that counts the occurences of each
%   chain code along the boundary of the region, where n is the number of
%   boundary curves. Only used if codeBoundary is true. See 
%   adaptive_chain_code for more. 
%
%   chainCodeBitCounts is a (1 x 8) matrix containing the apriori frequency
%   of each chain code. Only used if codeBoundary is true
%   Sigma is the sample mean of the feature vectors of the pixels in the
%   region.
%
%   pixelBitCount is the number of bits needed to code the coordinates of a
%   pixel in the image
%
%   codingLength is the computed coding length of boundary of the region 
%
%
%Authors: Shankar Rao, Hossein Mobahi
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign. 
%               University of California, Berkeley. 2009.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function codingLength = coding_length_chain(frequencies, chainCodeBitCounts, pixelBitCount)

curveCount = size(frequencies,1);

codingLength = ((pixelBitCount+3)*curveCount + sum(sum(frequencies,1).*chainCodeBitCounts))/2;
