% CODING_LENGTH_MEAN_COV coding length of samples from a Gaussian
% distribution using the sample mean and covariance. For data sets with a
% lot of samples relative to dimension, this should be much faster than
% coding_length
%
%   codingLength = coding_length_mean_cov(Sigma, sampleCount, 
%     totalSampleCount, epsilon2, affineModel, mu);
%
%   Sigma is the sample covariance of the given portion of the dataset
%
%   totalSampleCount is the total number of samples in the entire data set
%
%   sampleCount is the total number of samples in the given portion of the 
%     data set
%
%   epsilon2 is the square of the noise tolerance
%
%   affineModel is true if the coding length function should assume a
%   nonzero mean distribution, and false otherwise
%
%   mu is the sample mean of the given portion of the dataset.
%      Only used if affineModel is true.
%
%   codingLength is the computed coding length of the data set subject to
%   distortion epsilon^2
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign. 
%               University of California, Berkeley. 2008.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function codingLength = coding_length_mean_cov(Sigma, sampleCount, dimensionCount, epsilon2, affineModel, mu);


if nargin == 5 && ~affineModel,
    Sigma = Sigma + mu*mu';
end

if affineModel==false
    codingLength = ((dimensionCount + sampleCount)/2) * ...
        sum(log2(1+ (dimensionCount/epsilon2)*svd(Sigma))) ;       
else
    codingLength = ((dimensionCount + sampleCount)/2) * ...
        sum(log2(1+ (dimensionCount/epsilon2)*svd(Sigma)))...
        + (dimensionCount/2)*log2(1 + (mu.' * mu)/epsilon2);
end