function [outputImage gradientX gradientY] = construct_edge_image(inputImage, windowSize, useEdgeImage, parameters)

[imageX, imageY, channelCount] = size(inputImage);

imageDouble = double(inputImage);
warning off
if strcmp(parameters.colorMetric,'rgb')
    imageDouble = double(inputImage);
    channelCount = 3;
elseif strcmp(parameters.colorMetric,'lab')
    colorForm = makecform('srgb2lab');
    imageDouble = double(applycform(inputImage,colorForm));
    imageDouble = imageDouble(:,:,1:channelCount);
elseif strcmp(parameters.colorMetric,'uvl')
    colorForm = makecform('srgb2xyz');
    imageDouble = applycform(inputImage,colorForm);
    colorForm = makecform('xyz2uvl');
    imageDouble = double(applycform(imageDouble,colorForm));
    imageDouble = imageDouble(:,:,1:channelCount);
end
warning on

halfWindowSize = floor(windowSize/2);
GaussMask = torr_gauss_mask(halfWindowSize, parameters.GaussianVariance*halfWindowSize);

%outputImage = zeros(imageX, imageY);
if useEdgeImage,
    for channelIndex = 1:channelCount
        smoothedImage = conv2(imageDouble(:,:,channelIndex), GaussMask, 'same');
        gradientX = imfilter(smoothedImage, fspecial('sobel')/8, 'replicate');
        gradientY = imfilter(smoothedImage, fspecial('sobel')'/8, 'replicate');
        
        outputImage(:,:,channelIndex) = sqrt(gradientX.^2 + gradientY.^2);
    end
else
    smoothedImage = conv2(imageDouble(:,:,1), GaussMask, 'same');
    gradientX = imfilter(smoothedImage, fspecial('sobel')/8, 'replicate');
    gradientY = imfilter(smoothedImage, fspecial('sobel')'/8, 'replicate');
    
    outputImage = zeros(imageX, imageY, 2);
    outputImage(:,:,1) = gradientX;
    outputImage(:,:,2) = gradientY;
end


