% CONSTRUCT_MEAN_COV compute the sample mean and covariance of a given set 
%   of data
%
%   [mu, Sigma, n] = construct_mean_cov(data, groupIndices)
%
%   data is a (d x n) array where the each column is a sample vector
%
%   groupIndices is (1 x n) boolean array whose ith element is true if the
%   ith column vector of data should be included in the sample mean and
%   covariance
%
%   mu is the sample mean of the set of data
%     
%   Sigma is the sample covariance of the set of data
%
%   n is the number of samples in the set of data
%
%Authors: Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign. 
%               University of California, Berkeley. 2008.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function [mu, Sigma, n] = construct_mean_cov(data, groupIndices)
if nargin < 2
    groupIndices = true(1, size(data,2));
end
groupData = data(:, groupIndices);

mu = mean(groupData, 2);
n = length(groupIndices);
Sigma = (groupData*groupData')/n - (mu*mu');

