function [I,J]=doubleregion(img,id1,id2,ws)
% [I,J]=doubleregion(img,id1,id2,ws)
%
% Returns an array of the window centroids which
% only contain pixels of group "id1" and "id2", containing at least one
% pixel from each group
%
% img     A matrix indexed as img(i,j) containing integer values
%         indicating group ID of each point
% id1/id2 The two group ID's of interest
% ws      The window size (e.g. ws=15 means 15*15 windows)
% I       An array of location parameter "i" of the result
% J       An array of location parameter "j" of the result
%
%Authors: Hossein Mobahi, Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%
%(c) Copyright. University of Illinois at Urbana-Champaign. 
%               University of California, Berkeley. 2008.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.


I=[]; J=[];

% hws is the number of pixels between the center pixel and the edge of the
% window.
hws = (ws-1)/2;

for i=1+hws:size(img,1)-hws
    for j=1+hws:size(img,2)-hws
        FOUNDID1=false;
        FOUNDID2=false;
        ISINTERIOR=true;
        for ii=-hws:hws
            for jj=-hws:hws
                if img(i+ii,j+jj)==id1,
                    FOUNDID1=true;
                elseif img(i+ii,j+jj)==id2,    
                    FOUNDID2=true;
                else
                    ISINTERIOR=false;
                    break;
                end
            end
            if ~ISINTERIOR
                break;
            end
        end
        
        if ISINTERIOR && FOUNDID1 && FOUNDID2
            I=[I; i];
            J=[J; j];
        end
    end
end
