function [regions, pixelLabels, vicinityMap] = merge_edgeless_regions(regions, pixelLabels, vicinityMap, img, parameters, windowSizeArray, pixelFeatures, edgeImage,DEBUG)

initialRegionCount = length(vicinityMap);
[imageX, imageY] = size(pixelLabels);
windowSizeCount = length(windowSizeArray);


% Further merge neighboring segments whose boundaries do not correspond
% to strong edges

% Step 1: Compute edges
if strcmp(parameters.edgeMethod, 'line'),
    addpath('LineSegment/');
    [ignored, edgeMap] = LineSegment(img,'lowThreshold',0.05,'magnitudeThreshold',2);
else
    edgeImageX = edgeImage(:,:,1);
    edgeImageY = edgeImage(:,:,2);
end

% Step 2: Merge two neighbors whose boundary is not in the edge map
mergeIndices=[];

for regionIndex=1:initialRegionCount
    if isempty(vicinityMap{regionIndex})
        continue;
    end
    
    if strcmp(parameters.edgeMethod, 'line'),
        
        %for index = 1:initialRegionCount
        %    neighborRegionPixels{index} = zeros(1,2);
        %end
        pixelIndices = find(pixelLabels == regionIndex);
        if isempty(pixelIndices),
            continue;
        end
        neighborRegionPixels = zeros(2,initialRegionCount);
        for sampleIndex = 1:length(pixelIndices),
            
            [offsetX, offsetY] = ind2sub([imageX imageY], pixelIndices(sampleIndex));
            windowX1=max(offsetX-2,1);
            windowX2=min(offsetX+2,imageX);
            windowY1=max(offsetY-2,1);
            windowY2=min(offsetY+2,imageY);
            labelWindow = pixelLabels(windowX1:windowX2, windowY1:windowY2);
            % only consider a region a neighbor of this one if we have
            % not already visited it
            neighborArray = unique(labelWindow(labelWindow>regionIndex));
            if ~isempty(neighborArray),
                isOnEdgeMap = sum(sum(edgeMap(windowX1:windowX2, windowY1:windowY2)))>0;
                % Count boundary pixels
                neighborRegionPixels(1,neighborArray)=neighborRegionPixels(1, neighborArray)+1;
                if isOnEdgeMap,
                    % Count boundary pixels that on edgeMap
                    neighborRegionPixels(2,neighborArray)=neighborRegionPixels(2,neighborArray)+1;
                end
            end
            
        end
        for neighborIndex=1:initialRegionCount
            if neighborRegionPixels(1,neighborIndex)>0,
                % This is a neighbor segment
                if neighborRegionPixels(2,neighborIndex)/neighborRegionPixels(1,neighborIndex)<parameters.strongEdgeThreshold,
                    % Merge the two segments
                    mergeIndices(end+1,1:2)=[regionIndex, neighborIndex];
                end
            end
        end
    else
        for neighborIndex = 1:length(vicinityMap{regionIndex}),
            vicinityRegion = regions(vicinityMap{regionIndex}(neighborIndex));
            if vicinityRegion.label < regionIndex,
                continue;
            end
            edgeGradients = [];
            edgeGradientsCell = obtain_boundary_gradient(pixelLabels, regionIndex, vicinityRegion.label, edgeImageX, edgeImageY);
            % convert cell array to vector
            edgeGradients = [edgeGradients edgeGradientsCell{:}];
            edgeGradientsCell = obtain_boundary_gradient(pixelLabels, vicinityRegion.label, regionIndex, edgeImageX, edgeImageY);
            % convert cell array to vector
            edgeGradients = [edgeGradients edgeGradientsCell{:}];
            if ~isempty(edgeGradients) && median(abs(edgeGradients)) < parameters.edgeGradientThreshold,
                mergeIndices(end+1,1:2)=[regionIndex, vicinityRegion.label];
            end
        end
    end
end

regionIndices = 1:initialRegionCount;
for mergeIndex=1:size(mergeIndices,1)
    mergeFromIndex = max(regionIndices(mergeIndices(mergeIndex,:)));
    mergeToIndex = min(regionIndices(mergeIndices(mergeIndex,:)));
    if DEBUG>1,
        fprintf('Merging regions %d and %d that do not cross an edge boundary\n', mergeToIndex, mergeFromIndex);
    end
    if DEBUG>1
        testImage = zeros(imageX, imageY, 3);
        [xIndices, yIndices] = find(pixelLabels == mergeFromIndex | pixelLabels == mergeToIndex);
        for i=1:length(xIndices)
            if isa(img, 'uint8')
                testImage(xIndices(i),yIndices(i), :) = double(img(xIndices(i),yIndices(i),:))/255;
            else
                testImage(xIndices(i),yIndices(i), :) = double(img(xIndices(i),yIndices(i),:))/max(max(max(testImage)));
            end
        end
        figure(99)
        imshow(testImage);
    end
    if DEBUG>2
        pause
    end
    if mergeFromIndex~=mergeToIndex
        regionIndices(mergeIndices(mergeIndex,:)) = mergeToIndex;
        % Update region parameters
        mergedRegion = combine_joint_region_parameters( ...
            regions(mergeToIndex), regions(mergeFromIndex), pixelLabels, ...
            pixelFeatures, 1:windowSizeCount, windowSizeArray);
        regions(mergeToIndex) = mergedRegion;
        % Update labels;
        pixelLabels(pixelLabels == mergeFromIndex) = mergeToIndex;
        % Update vicinityMap
        for vicinityIndex=1:length(vicinityMap{mergeFromIndex})
            % Change all vicinityMap indices that pointing at 2 toward 1
            neighborIndex = vicinityMap{mergeFromIndex}(vicinityIndex);
            vicinityMap{neighborIndex} = union(vicinityMap{neighborIndex},mergeToIndex);
            vicinityMap{neighborIndex} = setdiff(vicinityMap{neighborIndex},mergeFromIndex);
        end
        vicinityMap{mergeToIndex} = setdiff(union(vicinityMap{mergeToIndex}, ...
            vicinityMap{mergeFromIndex}), [mergeToIndex mergeFromIndex]);
        vicinityMap{mergeFromIndex} = [];
    end
end
