% SEGIMAGE produce a version of an image where
% the edges between regions are highlighted
%
%   I = segImage(I,S,varargin)
%
%   I is the image whose edges are to be highlighted
%
%   S is an integer matrix that size of the image where (i,j)th
%   entry contains the group number of the given pixel.
%
%   See the optional parameters below
%
%
%Authors: Allen Yang, John Wright, Shankar Rao
%Contact: Shankar Rao <srrao@illinois.edu>
%(c) Copyright. University of Illinois at Urbana-Champaign. 2007.
%               University of California, Berkeley. 2007.
%
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.
function I = segmented_image(I,S,varargin)

boundaryWidth = 1;
boundaryColor = 'red';

% Parse the optional inputs.
if (mod(length(varargin), 2) ~= 0 ),
    error(['Extra Parameters passed to the function ''' mfilename ''' must be passed in pairs.']);
end
parameterCount = length(varargin)/2;

for parameterIndex = 1:parameterCount,
    parameterName = varargin{parameterIndex*2 - 1};
    parameterValue = varargin{parameterIndex*2};
    switch lower(parameterName)
        case 'boundarycolor'
            boundaryColor = parameterValue;
        case 'boundarywidth'
            boundaryWidth = parameterValue;
        otherwise
            error('Invalid parameter');
    end
end

[cx,cy] = gradient(S);
ccc = (abs(cx)+abs(cy))~=0;

if boundaryWidth>1
    boundaryWidth = ceil(boundaryWidth);
    dilateWindow = ones(boundaryWidth, boundaryWidth);
    ccc = imdilate(ccc,dilateWindow);
elseif boundaryWidth<1
    warning('boundaryWidth has been reset to 1.');
end

if strcmp(boundaryColor,'red')
    I(:,:,1) = I(:,:,1).*uint8(~ccc) + 255*uint8(ccc);
    I(:,:,2) = I(:,:,2).*uint8(~ccc) + 0*uint8(ccc);
    I(:,:,3) = I(:,:,3).*uint8(~ccc) + 0*uint8(ccc);
elseif strcmp(boundaryColor,'black')
    I(:,:,1) = min(I(:,:,1),~ccc);
    I(:,:,2) = min(I(:,:,2),~ccc);
    I(:,:,3) = min(I(:,:,3),~ccc);
else
    warning('Does not recognize boundaryColor other than red and black');
    I(:,:,1) = max(I(:,:,1),~ccc);
    I(:,:,2) = max(I(:,:,2),~ccc);
    I(:,:,3) = max(I(:,:,3),~ccc);
end

%I = mat2gray(I);