% VICINITY_MAP computes vicinityMap and register pixels in each segment in
% segmentRegistry
%
%   [vicinityMap,segmentRegistry]=vicinity_map(pixelLabels,isDirectedMap)
%
%   pixelLabels is an array the size of the image that contains the label
%   of every pixel (which region each pixel belongs to)
%
%   isDirectedMap is true if the vicinity map is a directed graph, and is
%   false otherwise
%
%   vicinityMap is a (1 x r) cell array where the ith entry contains the
%   labels of regions adjacent to the ith region. r is the total number of
%   regions.
%
%   segmentRegistry is a (1 x r) cell array where the ith entry contains
%   the linear indices of each pixel in the ith region.
%
%Authors: John Wright, and Allen Y. Yang
%Contact: Allen Y. Yang <yang@eecs.berkeley.edu>
%
%(c) Copyright. University of California, Berkeley. 2007.
%
%Notice: The packages should NOT be used for any commercial purposes
%without direct consent of their author(s). The authors are not responsible
%for any potential property loss or damage caused directly or indirectly by
%the usage of the software.

function vicinityMap = vicinity_map_fast(pixelLabels)
% vicinity_map computes vicinityMap and register pixels in each segment in
% segmentRegistry
%
% Note: the input parameter pixelLabels must have the dimension imageX*imageY.




regionCount = max(max(pixelLabels));

vicinityMap = cell(1,regionCount);

[xGradient,yGradient] = gradient(pixelLabels);
boundaryMap = (abs(xGradient)+abs(yGradient))~=0;


neighborHood = [0 1 0; 1 0 1; 0 1 0];
for regionIndex = 1:regionCount
    onBoundary = boundaryMap & (pixelLabels==regionIndex);
    nearBoundary = imdilate(onBoundary, neighborHood) & ~onBoundary;
    inVicinity = false(1,regionCount);
    inVicinity(unique(pixelLabels(nearBoundary))) = true;
    inVicinity(regionIndex) = false;
    vicinityMap{regionIndex} = find(inVicinity);
end

% pixelIndex = 0;
% for yIndex=1:imageY
%     for xIndex=1:imageX
%         
%         pixelIndex = pixelIndex + 1;
%         currentPixelLabel = pixelLabels(xIndex,yIndex);
%         
%         if boundaryMap(xIndex,yIndex)==true
%             % update vicinityMap
%             for offsetIndex=1:4,%8
%                 xxIndex = xIndex + offsetMatrix(1,offsetIndex);
%                 yyIndex = yIndex + offsetMatrix(2,offsetIndex);
%                 if (xxIndex>0) && (xxIndex<=imageX) && (yyIndex>0) && (yyIndex<=imageY) && ...
%                         (pixelLabels(xxIndex,yyIndex)~=currentPixelLabel) && ...
%                         ((isDirectedMap==false) | (pixelLabels(xxIndex,yyIndex)>currentPixelLabel))
%                         vicinityMap{currentPixelLabel} = union(vicinityMap{currentPixelLabel},pixelLabels(xxIndex,yyIndex));
%                         vicinityMap{pixelLabels(xxIndex,yyIndex)} = union(vicinityMap{pixelLabels(xxIndex,yyIndex)},currentPixelLabel);
%                 end
%             end
%         end
%         
% 
%     end
% end